==> cirq-0.4.0.dev4220181119182308-py2-none-any.whl: cirq/ops/parity_gates_test.py <==
# coding=utf-8
# Copyright 2018 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

u"""Tests for `parity_gates.py`."""

from __future__ import division
from __future__ import absolute_import
import numpy as np

import cirq


def test_xx_init():
    assert cirq.XXPowGate(exponent=1).exponent == 1
    v = cirq.XXPowGate(exponent=0.5)
    assert v.exponent == 0.5


def test_xx_eq():
    eq = cirq.testing.EqualsTester()
    eq.add_equality_group(cirq.XX,
                          cirq.XXPowGate(),
                          cirq.XXPowGate(exponent=1, global_shift=0),
                          cirq.XXPowGate(exponent=3, global_shift=0))
    eq.add_equality_group(cirq.XX**0.5, cirq.XX**2.5, cirq.XX**4.5)
    eq.add_equality_group(cirq.XX**0.25, cirq.XX**2.25, cirq.XX**-1.75)

    iXX = cirq.XXPowGate(global_shift=0.5)
    eq.add_equality_group(iXX**0.5, iXX**4.5)
    eq.add_equality_group(iXX**2.5, iXX**6.5)


def test_xx_pow():
    assert cirq.XX**0.5 != cirq.XX**-0.5
    assert cirq.XX**-1 == cirq.XX
    assert (cirq.XX**-1)**0.5 == cirq.XX**-0.5


def test_xx_str():
    assert unicode(cirq.XX) == u'XX'
    assert unicode(cirq.XX**0.5) == u'XX**0.5'
    assert unicode(cirq.XXPowGate(global_shift=0.1)) == u'XX'

    ms = cirq.XXPowGate(global_shift=-0.5)
    assert unicode(ms) == u'MS(π/2)'
    assert unicode(ms**2) == u'MS(2.0π/2)'
    assert unicode(ms**-1) == u'MS(-1.0π/2)'


def test_xx_repr():
    assert repr(cirq.XXPowGate()) == u'cirq.XX'
    assert repr(cirq.XXPowGate(exponent=0.5)) == u'(cirq.XX**0.5)'

    cirq.testing.assert_equivalent_repr(cirq.XX)
    cirq.testing.assert_equivalent_repr(cirq.XX**0.1)
    cirq.testing.assert_equivalent_repr(cirq.XX**0.5)
    cirq.testing.assert_equivalent_repr(cirq.XX**2)
    cirq.testing.assert_equivalent_repr(cirq.XXPowGate(exponent=0.2,
                                                       global_shift=0.3))

    ms = cirq.XXPowGate(global_shift=-0.5)
    assert (repr(ms) == u'cirq.MS(np.pi/2)')
    assert (repr(ms**2) == u'cirq.MS(2.0*np.pi/2)')
    assert (repr(ms**-0.5) == u'cirq.MS(-0.5*np.pi/2)')

    cirq.testing.assert_equivalent_repr(ms)
    cirq.testing.assert_equivalent_repr(ms**0.1)
    cirq.testing.assert_equivalent_repr(ms**0.5)
    cirq.testing.assert_equivalent_repr(ms**2)


def test_xx_matrix():
    np.testing.assert_allclose(cirq.unitary(cirq.XX),
                               np.array([[0, 0, 0, 1],
                                         [0, 0, 1, 0],
                                         [0, 1, 0, 0],
                                         [1, 0, 0, 0]]),
                               atol=1e-8)
    np.testing.assert_allclose(cirq.unitary(cirq.XX**2),
                               np.eye(4),
                               atol=1e-8)
    c = np.cos(np.pi/6)
    s = -1j*np.sin(np.pi/6)
    np.testing.assert_allclose(
        cirq.unitary(cirq.XXPowGate(exponent=1/3,
                                    global_shift=-0.5)),
        np.array([[c, 0, 0, s],
                  [0, c, s, 0],
                  [0, s, c, 0],
                  [s, 0, 0, c]]),
        atol=1e-8)


def test_xx_consistent():
    cirq.testing.assert_eigen_gate_has_consistent_apply_unitary(cirq.XXPowGate)

    cases = [cirq.XX,
             cirq.XX**0.1,
             cirq.XXPowGate(global_shift=0.1, exponent=0.2),
             cirq.XXPowGate(global_shift=-0.5, exponent=0.4)]
    for case in cases:
        cirq.testing.assert_qasm_is_consistent_with_unitary(case)
        cirq.testing.assert_decompose_is_consistent_with_unitary(case)
        cirq.testing.assert_phase_by_is_consistent_with_unitary(case)


def test_xx_diagrams():
    a = cirq.NamedQubit(u'a')
    b = cirq.NamedQubit(u'b')
    circuit = cirq.Circuit.from_ops(
        cirq.XX(a, b),
        cirq.XX(a, b)**3,
        cirq.XX(a, b)**0.5,
    )
    cirq.testing.assert_has_diagram(circuit, u"""
a: ───XX───XX───XX───────
      │    │    │
b: ───XX───XX───XX^0.5───
""")


def test_yy_init():
    assert cirq.YYPowGate(exponent=1).exponent == 1
    v = cirq.YYPowGate(exponent=0.5)
    assert v.exponent == 0.5


def test_yy_eq():
    eq = cirq.testing.EqualsTester()
    eq.add_equality_group(cirq.YY,
                          cirq.YYPowGate(),
                          cirq.YYPowGate(exponent=1, global_shift=0),
                          cirq.YYPowGate(exponent=3, global_shift=0))
    eq.add_equality_group(cirq.YY**0.5, cirq.YY**2.5, cirq.YY**4.5)
    eq.add_equality_group(cirq.YY**0.25, cirq.YY**2.25, cirq.YY**-1.75)

    iYY = cirq.YYPowGate(global_shift=0.5)
    eq.add_equality_group(iYY**0.5, iYY**4.5)
    eq.add_equality_group(iYY**2.5, iYY**6.5)


def test_yy_pow():
    assert cirq.YY**0.5 != cirq.YY**-0.5
    assert cirq.YY**-1 == cirq.YY
    assert (cirq.YY**-1)**0.5 == cirq.YY**-0.5


def test_yy_str():
    assert unicode(cirq.YY) == u'YY'
    assert unicode(cirq.YY**0.5) == u'YY**0.5'
    assert unicode(cirq.YYPowGate(global_shift=0.1)) == u'YY'

    iYY = cirq.YYPowGate(global_shift=0.5)
    assert unicode(iYY) == u'YY'


def test_yy_repr():
    assert repr(cirq.YYPowGate()) == u'cirq.YY'
    assert repr(cirq.YYPowGate(exponent=0.5)) == u'(cirq.YY**0.5)'

    cirq.testing.assert_equivalent_repr(cirq.YY)
    cirq.testing.assert_equivalent_repr(cirq.YY**0.1)
    cirq.testing.assert_equivalent_repr(cirq.YY**0.5)
    cirq.testing.assert_equivalent_repr(cirq.YY**2)
    cirq.testing.assert_equivalent_repr(cirq.YYPowGate(exponent=0.2,
                                                       global_shift=0.3))


def test_yy_matrix():
    np.testing.assert_allclose(cirq.unitary(cirq.YY),
                               np.array([[0, 0, 0, -1],
                                         [0, 0, 1, 0],
                                         [0, 1, 0, 0],
                                         [-1, 0, 0, 0]]),
                               atol=1e-8)
    np.testing.assert_allclose(cirq.unitary(cirq.YY**2),
                               np.eye(4),
                               atol=1e-8)
    c = np.cos(np.pi/6)
    s = 1j*np.sin(np.pi/6)
    np.testing.assert_allclose(
        cirq.unitary(cirq.YYPowGate(exponent=1/3,
                                    global_shift=-0.5)),
        np.array([[c, 0, 0, s],
                  [0, c, -s, 0],
                  [0, -s, c, 0],
                  [s, 0, 0, c]]),
        atol=1e-8)


def test_yy_consistent():
    cirq.testing.assert_eigen_gate_has_consistent_apply_unitary(cirq.YYPowGate)

    cases = [cirq.YY,
             cirq.YY**0.1,
             cirq.YYPowGate(global_shift=0.1, exponent=0.2),
             cirq.YYPowGate(global_shift=-0.5, exponent=0.4)]
    for case in cases:
        cirq.testing.assert_qasm_is_consistent_with_unitary(case)
        cirq.testing.assert_decompose_is_consistent_with_unitary(case)
        cirq.testing.assert_phase_by_is_consistent_with_unitary(case)


def test_yy_diagrams():
    a = cirq.NamedQubit(u'a')
    b = cirq.NamedQubit(u'b')
    circuit = cirq.Circuit.from_ops(
        cirq.YY(a, b),
        cirq.YY(a, b)**3,
        cirq.YY(a, b)**0.5,
    )
    cirq.testing.assert_has_diagram(circuit, u"""
a: ───YY───YY───YY───────
      │    │    │
b: ───YY───YY───YY^0.5───
""")


def test_zz_init():
    assert cirq.ZZPowGate(exponent=1).exponent == 1
    v = cirq.ZZPowGate(exponent=0.5)
    assert v.exponent == 0.5


def test_zz_eq():
    eq = cirq.testing.EqualsTester()
    eq.add_equality_group(cirq.ZZ,
                          cirq.ZZPowGate(),
                          cirq.ZZPowGate(exponent=1, global_shift=0),
                          cirq.ZZPowGate(exponent=3, global_shift=0))
    eq.add_equality_group(cirq.ZZ**0.5, cirq.ZZ**2.5, cirq.ZZ**4.5)
    eq.add_equality_group(cirq.ZZ**0.25, cirq.ZZ**2.25, cirq.ZZ**-1.75)

    iZZ = cirq.ZZPowGate(global_shift=0.5)
    eq.add_equality_group(iZZ**0.5, iZZ**4.5)
    eq.add_equality_group(iZZ**2.5, iZZ**6.5)


def test_zz_pow():
    assert cirq.ZZ**0.5 != cirq.ZZ**-0.5
    assert cirq.ZZ**-1 == cirq.ZZ
    assert (cirq.ZZ**-1)**0.5 == cirq.ZZ**-0.5


def test_zz_str():
    assert unicode(cirq.ZZ) == u'ZZ'
    assert unicode(cirq.ZZ**0.5) == u'ZZ**0.5'
    assert unicode(cirq.ZZPowGate(global_shift=0.1)) == u'ZZ'

    iZZ = cirq.ZZPowGate(global_shift=0.5)
    assert unicode(iZZ) == u'ZZ'


def test_zz_repr():
    assert repr(cirq.ZZPowGate()) == u'cirq.ZZ'
    assert repr(cirq.ZZPowGate(exponent=0.5)) == u'(cirq.ZZ**0.5)'

    cirq.testing.assert_equivalent_repr(cirq.ZZ)
    cirq.testing.assert_equivalent_repr(cirq.ZZ**0.1)
    cirq.testing.assert_equivalent_repr(cirq.ZZ**0.5)
    cirq.testing.assert_equivalent_repr(cirq.ZZ**2)
    cirq.testing.assert_equivalent_repr(cirq.ZZPowGate(exponent=0.2,
                                                       global_shift=0.3))


def test_zz_matrix():
    np.testing.assert_allclose(cirq.unitary(cirq.ZZ),
                               np.array([[1, 0, 0, 0],
                                         [0, -1, 0, 0],
                                         [0, 0, -1, 0],
                                         [0, 0, 0, 1]]),
                               atol=1e-8)
    np.testing.assert_allclose(cirq.unitary(cirq.ZZ**2),
                               np.eye(4),
                               atol=1e-8)
    b = 1j**0.25
    a = np.conj(b)
    np.testing.assert_allclose(
        cirq.unitary(cirq.ZZPowGate(exponent=0.25,
                                    global_shift=-0.5)),
        np.array([[a, 0, 0, 0],
                  [0, b, 0, 0],
                  [0, 0, b, 0],
                  [0, 0, 0, a]]),
        atol=1e-8)


def test_zz_consistent():
    cirq.testing.assert_eigen_gate_has_consistent_apply_unitary(cirq.ZZPowGate)

    cases = [cirq.ZZ,
             cirq.ZZ**0.1,
             cirq.ZZPowGate(global_shift=0.1, exponent=0.2),
             cirq.ZZPowGate(global_shift=-0.5, exponent=0.4)]
    for case in cases:
        cirq.testing.assert_qasm_is_consistent_with_unitary(case)
        cirq.testing.assert_decompose_is_consistent_with_unitary(case)
        cirq.testing.assert_phase_by_is_consistent_with_unitary(case)


def test_zz_diagrams():
    a = cirq.NamedQubit(u'a')
    b = cirq.NamedQubit(u'b')
    circuit = cirq.Circuit.from_ops(
        cirq.ZZ(a, b),
        cirq.ZZ(a, b)**3,
        cirq.ZZ(a, b)**0.5,
    )
    cirq.testing.assert_has_diagram(circuit, u"""
a: ───ZZ───ZZ───ZZ───────
      │    │    │
b: ───ZZ───ZZ───ZZ^0.5───
""")

==> cirq-0.5.0-py2-none-any.whl: cirq/ops/parity_gates_test.py <==
# coding=utf-8
# Copyright 2018 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

u"""Tests for `parity_gates.py`."""

from __future__ import division
from __future__ import absolute_import
import numpy as np
import pytest

import cirq


@pytest.mark.parametrize(u'eigen_gate_type', [
    cirq.XXPowGate,
    cirq.YYPowGate,
    cirq.ZZPowGate,
    ]
)
def test_eigen_gates_consistent_protocols(eigen_gate_type):
    cirq.testing.assert_eigengate_implements_consistent_protocols(
            eigen_gate_type)


def test_xx_init():
    assert cirq.XXPowGate(exponent=1).exponent == 1
    v = cirq.XXPowGate(exponent=0.5)
    assert v.exponent == 0.5


def test_xx_eq():
    eq = cirq.testing.EqualsTester()
    eq.add_equality_group(cirq.XX,
                          cirq.XXPowGate(),
                          cirq.XXPowGate(exponent=1, global_shift=0),
                          cirq.XXPowGate(exponent=3, global_shift=0))
    eq.add_equality_group(cirq.XX**0.5, cirq.XX**2.5, cirq.XX**4.5)
    eq.add_equality_group(cirq.XX**0.25, cirq.XX**2.25, cirq.XX**-1.75)

    iXX = cirq.XXPowGate(global_shift=0.5)
    eq.add_equality_group(iXX**0.5, iXX**4.5)
    eq.add_equality_group(iXX**2.5, iXX**6.5)


def test_xx_pow():
    assert cirq.XX**0.5 != cirq.XX**-0.5
    assert cirq.XX**-1 == cirq.XX
    assert (cirq.XX**-1)**0.5 == cirq.XX**-0.5


def test_xx_str():
    assert unicode(cirq.XX) == u'XX'
    assert unicode(cirq.XX**0.5) == u'XX**0.5'
    assert unicode(cirq.XXPowGate(global_shift=0.1)) == u'XX'

    ms = cirq.XXPowGate(global_shift=-0.5)
    assert unicode(ms) == u'MS(π/2)'
    assert unicode(ms**2) == u'MS(2.0π/2)'
    assert unicode(ms**-1) == u'MS(-1.0π/2)'


def test_xx_repr():
    assert repr(cirq.XXPowGate()) == u'cirq.XX'
    assert repr(cirq.XXPowGate(exponent=0.5)) == u'(cirq.XX**0.5)'

    ms = cirq.XXPowGate(global_shift=-0.5)
    assert (repr(ms) == u'cirq.MS(np.pi/2)')
    assert (repr(ms**2) == u'cirq.MS(2.0*np.pi/2)')
    assert (repr(ms**-0.5) == u'cirq.MS(-0.5*np.pi/2)')


def test_xx_matrix():
    np.testing.assert_allclose(cirq.unitary(cirq.XX),
                               np.array([[0, 0, 0, 1],
                                         [0, 0, 1, 0],
                                         [0, 1, 0, 0],
                                         [1, 0, 0, 0]]),
                               atol=1e-8)
    np.testing.assert_allclose(cirq.unitary(cirq.XX**2),
                               np.eye(4),
                               atol=1e-8)
    c = np.cos(np.pi/6)
    s = -1j*np.sin(np.pi/6)
    np.testing.assert_allclose(
        cirq.unitary(cirq.XXPowGate(exponent=1/3,
                                    global_shift=-0.5)),
        np.array([[c, 0, 0, s],
                  [0, c, s, 0],
                  [0, s, c, 0],
                  [s, 0, 0, c]]),
        atol=1e-8)


def test_xx_diagrams():
    a = cirq.NamedQubit(u'a')
    b = cirq.NamedQubit(u'b')
    circuit = cirq.Circuit.from_ops(
        cirq.XX(a, b),
        cirq.XX(a, b)**3,
        cirq.XX(a, b)**0.5,
    )
    cirq.testing.assert_has_diagram(circuit, u"""
a: ───XX───XX───XX───────
      │    │    │
b: ───XX───XX───XX^0.5───
""")


def test_yy_init():
    assert cirq.YYPowGate(exponent=1).exponent == 1
    v = cirq.YYPowGate(exponent=0.5)
    assert v.exponent == 0.5


def test_yy_eq():
    eq = cirq.testing.EqualsTester()
    eq.add_equality_group(cirq.YY,
                          cirq.YYPowGate(),
                          cirq.YYPowGate(exponent=1, global_shift=0),
                          cirq.YYPowGate(exponent=3, global_shift=0))
    eq.add_equality_group(cirq.YY**0.5, cirq.YY**2.5, cirq.YY**4.5)
    eq.add_equality_group(cirq.YY**0.25, cirq.YY**2.25, cirq.YY**-1.75)

    iYY = cirq.YYPowGate(global_shift=0.5)
    eq.add_equality_group(iYY**0.5, iYY**4.5)
    eq.add_equality_group(iYY**2.5, iYY**6.5)


def test_yy_pow():
    assert cirq.YY**0.5 != cirq.YY**-0.5
    assert cirq.YY**-1 == cirq.YY
    assert (cirq.YY**-1)**0.5 == cirq.YY**-0.5


def test_yy_str():
    assert unicode(cirq.YY) == u'YY'
    assert unicode(cirq.YY**0.5) == u'YY**0.5'
    assert unicode(cirq.YYPowGate(global_shift=0.1)) == u'YY'

    iYY = cirq.YYPowGate(global_shift=0.5)
    assert unicode(iYY) == u'YY'


def test_yy_repr():
    assert repr(cirq.YYPowGate()) == u'cirq.YY'
    assert repr(cirq.YYPowGate(exponent=0.5)) == u'(cirq.YY**0.5)'


def test_yy_matrix():
    np.testing.assert_allclose(cirq.unitary(cirq.YY),
                               np.array([[0, 0, 0, -1],
                                         [0, 0, 1, 0],
                                         [0, 1, 0, 0],
                                         [-1, 0, 0, 0]]),
                               atol=1e-8)
    np.testing.assert_allclose(cirq.unitary(cirq.YY**2),
                               np.eye(4),
                               atol=1e-8)
    c = np.cos(np.pi/6)
    s = 1j*np.sin(np.pi/6)
    np.testing.assert_allclose(
        cirq.unitary(cirq.YYPowGate(exponent=1/3,
                                    global_shift=-0.5)),
        np.array([[c, 0, 0, s],
                  [0, c, -s, 0],
                  [0, -s, c, 0],
                  [s, 0, 0, c]]),
        atol=1e-8)


def test_yy_diagrams():
    a = cirq.NamedQubit(u'a')
    b = cirq.NamedQubit(u'b')
    circuit = cirq.Circuit.from_ops(
        cirq.YY(a, b),
        cirq.YY(a, b)**3,
        cirq.YY(a, b)**0.5,
    )
    cirq.testing.assert_has_diagram(circuit, u"""
a: ───YY───YY───YY───────
      │    │    │
b: ───YY───YY───YY^0.5───
""")


def test_zz_init():
    assert cirq.ZZPowGate(exponent=1).exponent == 1
    v = cirq.ZZPowGate(exponent=0.5)
    assert v.exponent == 0.5


def test_zz_eq():
    eq = cirq.testing.EqualsTester()
    eq.add_equality_group(cirq.ZZ,
                          cirq.ZZPowGate(),
                          cirq.ZZPowGate(exponent=1, global_shift=0),
                          cirq.ZZPowGate(exponent=3, global_shift=0))
    eq.add_equality_group(cirq.ZZ**0.5, cirq.ZZ**2.5, cirq.ZZ**4.5)
    eq.add_equality_group(cirq.ZZ**0.25, cirq.ZZ**2.25, cirq.ZZ**-1.75)

    iZZ = cirq.ZZPowGate(global_shift=0.5)
    eq.add_equality_group(iZZ**0.5, iZZ**4.5)
    eq.add_equality_group(iZZ**2.5, iZZ**6.5)


def test_zz_pow():
    assert cirq.ZZ**0.5 != cirq.ZZ**-0.5
    assert cirq.ZZ**-1 == cirq.ZZ
    assert (cirq.ZZ**-1)**0.5 == cirq.ZZ**-0.5


def test_zz_str():
    assert unicode(cirq.ZZ) == u'ZZ'
    assert unicode(cirq.ZZ**0.5) == u'ZZ**0.5'
    assert unicode(cirq.ZZPowGate(global_shift=0.1)) == u'ZZ'

    iZZ = cirq.ZZPowGate(global_shift=0.5)
    assert unicode(iZZ) == u'ZZ'


def test_zz_repr():
    assert repr(cirq.ZZPowGate()) == u'cirq.ZZ'
    assert repr(cirq.ZZPowGate(exponent=0.5)) == u'(cirq.ZZ**0.5)'


def test_zz_matrix():
    np.testing.assert_allclose(cirq.unitary(cirq.ZZ),
                               np.array([[1, 0, 0, 0],
                                         [0, -1, 0, 0],
                                         [0, 0, -1, 0],
                                         [0, 0, 0, 1]]),
                               atol=1e-8)
    np.testing.assert_allclose(cirq.unitary(cirq.ZZ**2),
                               np.eye(4),
                               atol=1e-8)
    b = 1j**0.25
    a = np.conj(b)
    np.testing.assert_allclose(
        cirq.unitary(cirq.ZZPowGate(exponent=0.25,
                                    global_shift=-0.5)),
        np.array([[a, 0, 0, 0],
                  [0, b, 0, 0],
                  [0, 0, b, 0],
                  [0, 0, 0, a]]),
        atol=1e-8)


def test_zz_diagrams():
    a = cirq.NamedQubit(u'a')
    b = cirq.NamedQubit(u'b')
    circuit = cirq.Circuit.from_ops(
        cirq.ZZ(a, b),
        cirq.ZZ(a, b)**3,
        cirq.ZZ(a, b)**0.5,
    )
    cirq.testing.assert_has_diagram(circuit, u"""
a: ───ZZ───ZZ───ZZ───────
      │    │    │
b: ───ZZ───ZZ───ZZ^0.5───
""")
